import QtQuick 1.1

// This element manages audio priority and tracks requests from audio sources to play audio to user.

// The AudioManager accepts requests to play audio via several xxxRequest functions, where xxx is ipod, tv, va, etc.
// It responds by modifying a set of xxxAllowed properties, which indicate that it is OK for the corresponding source to play audio.
// The ipod source is coded as the highest priority source, and if playing, all other sources are denied.

Item {
    id: audioManager

    property int volume: 0
    property bool mute: false
    property bool btconnected: false
    property bool ipodPlaying: false
    property bool ipodPlayingVideo: false
    property bool tvPlaying: false
    property bool virtActivePlaying: false
    property bool netpulsePlaying: false
    property bool flashPlaying: false

    property bool ipodAllowed: false
    property bool tvAllowed: false
    property bool virtActiveAllowed: false
    property bool netpulseAllowed: false
    property bool ipodVideoAllowed: false
    property bool tvVideoAllowed: false
    property bool virtActiveVideoAllowed: false
    property bool netpulseVideoAllowed: false
    property bool rcaAllowed: false
    property bool flashAllowed: false
    property bool btAllowed: false
    property string currentMuxSource: ""

    // if there are multiple sources, ipod may be unselected to hear another
    property bool musicSelected: false

    property variant audioSources: []
    property string videoSource: ""

    property bool audioPlaying: audioSources.length > 0
    property bool debug: false

    function debugPrint(message) {
        if(debug)
            errorManager.debugMessage("AudioManager: " + message)
    }

    function reset() {
        debugPrint("reset")
        musicSelected = false

        ipodAllowed = false
        virtActiveAllowed = false
        netpulseAllowed = false
        tvAllowed = false
        rcaAllowed = false

        virtActiveVideoAllowed = false
        netpulseVideoAllowed = false
        ipodVideoAllowed = false
        tvVideoAllowed = false
        flashAllowed = false

        btAllowed = false

        setMux("none")
        audioManager.volume = 24

        rebuildSourceList()
    }

    function setVolume(percent) {
        audioManager.volume = percent
    }

    function setMute(muted) {
        audioManager.mute = muted
    }

    onMuteChanged: {
        if(mute)
            brainboard.headphoneVolume = 0
        else
            brainboard.headphoneVolume = audioManager.volume
    }

    onVolumeChanged: {
        audioManager.mute = false
        debugPrint("volume changed:" + volume)
        brainboard.headphoneVolume = volume
    }

    function setMux(source) {
        switch(source) {
            case "none":{
                if(currentMuxSource != "none") {
                    brainboard.setNoneAVSource()
                    currentMuxSource = "none"
                }
                break;
            }
            case "ipod":{
                if(currentMuxSource != "ipod") {
                    brainboard.setiPodAVSource()
                    currentMuxSource = "ipod"
                }
                break;
            }
            case "rca":{
                if(currentMuxSource != "rca") {
                    brainboard.setAUXAVSource()
                    currentMuxSource = "rca"
                }
                break;
            }
        }
    }

    function ipodRequest() {
        debugPrint("ipod request")
        if(!ipodAllowed)
            enableSource("ipod")
    }

    function ipodVideoRequest() {
        debugPrint("ipod video request")
        if(!ipodVideoAllowed) {
            enableSource("ipod")
            tvVideoAllowed = false
            virtActiveVideoAllowed = false
            netpulseVideoAllowed = false
            ipodVideoAllowed = true
            flashAllowed = false
            videoSource = "ipod"
        }
    }

    function tvRequest() {
        debugPrint("tv request")
        _lastRequested = "tv"
        if(!tvAllowed)
            enableSource("tv")
    }

    function vaRequest() {
        debugPrint("va request")
        _lastRequested = "va"
        if(!virtActiveAllowed)
            enableSource("va")
    }

    function netpulseRequest() {
        _lastRequested = "netpulse"
        if(!netpulseAllowed)
            enableSource("netpulse")
    }

    function rcaRequest() {
        _lastRequested = "rca"
        if(!rcaAllowed)
            enableSource("rca")
    }

    function flashRequest() {
        _lastRequested = "flash"
        if(!flashAllowed)
            enableSource("flash")
    }

    function btRequest() {
        _lastRequested = "bt"
        if(!btAllowed)
            enableSource("bt")
    }

    Component.onCompleted: reset()

    property string _lastRequested   // string name of last source which will be re-enabled when ipod stops

    onTvPlayingChanged: rebuildSourceList()
    onVirtActivePlayingChanged: rebuildSourceList()
    onNetpulsePlayingChanged: rebuildSourceList()
    onIpodPlayingChanged: rebuildSourceList()
    onBtconnectedChanged: rebuildSourceList()
    onFlashPlayingChanged: rebuildSourceList()

    onVideoSourceChanged: debugPrint("Video Source:" + videoSource)

    onMusicSelectedChanged: {
        // the "source" button was toggled
        debugPrint("ipod selected: " + musicSelected)
        if(musicSelected) {
            if (btconnected) {
                enableSource("bt")
            } else {
                enableSource("ipod")
            }
        }
        else { // re-enable video source that may be playing
            debugPrint("re-enable last source:" + _lastRequested)
            enableSource(_lastRequested)
        }
    }

    function enableSource(name) {
        switch(name) {
            case "ipod": {
                tvAllowed = false
                virtActiveAllowed = false
                netpulseAllowed = false
                ipodAllowed = true
                musicSelected = true
                flashAllowed = false
                btAllowed = false

                setMux("ipod")
                break;
            }
            case "tv": {
                musicSelected = false
                ipodAllowed = false
                virtActiveAllowed = false
                netpulseAllowed = false
                tvAllowed = true
                virtActiveVideoAllowed = false
                netpulseVideoAllowed = false
                ipodVideoAllowed = false
                tvVideoAllowed = true
                flashAllowed = false
                btAllowed = false

                videoSource = "tv"

                if(tv.isAuxInput)
                    setMux("rca")
                else
                    setMux("none")

                break;
            }
            case "rca": {
                musicSelected = false
                ipodAllowed = false
                virtActiveAllowed = false
                netpulseAllowed = false
                tvAllowed = true
                virtActiveVideoAllowed = false
                netpulseVideoAllowed = false
                ipodVideoAllowed = false
                tvVideoAllowed = false
                rcaAllowed = true
                flashAllowed = false
                btAllowed = false

                videoSource = "rca"

                setMux("rca")

                break;
            }
            case "va": {
                musicSelected = false
                ipodAllowed = false
                tvAllowed = false
                netpulseAllowed = false
                virtActiveAllowed = true
                netpulseVideoAllowed = false
                ipodVideoAllowed = false
                tvVideoAllowed = false
                virtActiveVideoAllowed = true
                flashAllowed = false
                btAllowed = false

                videoSource = "va"

                setMux("none")

                break;
            }
            case "netpulse": {
                musicSelected = false
                ipodAllowed = false
                tvAllowed = false
                virtActiveAllowed = false
                netpulseAllowed = true
                ipodVideoAllowed = false
                tvVideoAllowed = false
                virtActiveVideoAllowed = false
                netpulseVideoAllowed = true
                flashAllowed = false
                btAllowed = false

                videoSource = "netpulse"

                setMux("none")

                break;
            }
            case "flash": {
                musicSelected = false
                ipodAllowed = false
                tvAllowed = false
                virtActiveAllowed = false
                netpulseAllowed = true
                ipodVideoAllowed = false
                tvVideoAllowed = false
                virtActiveVideoAllowed = false
                netpulseVideoAllowed = false
                flashAllowed = true
                btAllowed = false

//                videoSource = "netpulse" // set to flash?

                setMux("none")

                break;
            }

            case "bt": {
                musicSelected = true
                ipodAllowed = false
                tvAllowed = false
                netpulseAllowed = false
                virtActiveAllowed = false
                flashAllowed = false
                btAllowed = true

                setMux("none")
                break
            }
        }
    }

    // update string-based list of currently playing sources
    function rebuildSourceList() {
        var newSources = ""
        newSources = []
        if(ipodPlaying)
            newSources = newSources.concat(["ipod"])
        if(btconnected)
            newSources = newSources.concat(["bt"])
        if(tvPlaying)
            newSources = newSources.concat(["tv"])
        else if(videoSource == "tv"){
            videoSource = ""
        }
        if(virtActivePlaying)
            newSources = newSources.concat(["va"])
        else if(videoSource == "va") {
            videoSource = ""
        }
        if(netpulsePlaying)
            newSources = newSources.concat(["netpulse"])
        else if(videoSource == "netpulse") {
            videoSource = ""
        }
        if (flashPlaying)
            newSources = newSources.concat(["flash"])

        audioSources = newSources
    }
}
