// import QtQuick 1.0 // to target S60 5th Edition or Maemo 5
import QtQuick 1.1
import WorkoutRunData 1.0
import ".."
import "Globals.js" as Globals

Item {
    id: root
    property variant viewContainer
    property bool cooldownEnabled: workoutProgram.cooldownEnabled
    property bool bikeModeAvailable: workoutProgram.bikeModeAvailable
    property string units: workoutScreen.unitState
    property bool isSelected: viewContainer.currentViewName === "TOOLS_VIEW"

    signal cooldownPressed()
    signal unitsPressed()
    signal hrcCruisePressed()
    signal bikeModePressed()
    signal fanButtonPressed()

    onFanButtonPressed: {
        var newspeed = (brainboard.fanSpeed === 100 ? 0 : 100)
        console.log("fan speed is", brainboard.fanSpeed, "; changing to", newspeed)
        var result = brainboard.requestChangeFanSpeed(newspeed)
    }

    function unitsNext() {
        if (root.units === "METRIC") {
            root.units = "ENGLISH";
        } else {
            root.units = "METRIC";
        }
    }

    onIsSelectedChanged: {
        if (isSelected)
            toolsMenu.currentIndex = 0
    }

    onCooldownPressed: workoutManager.startCooldown()
    onUnitsPressed: {
        unitsNext();
        workoutScreen.unitState = root.units;
    }

    onHrcCruisePressed: {
        if (heartRateCenter.hrCruiseEnabled) {
            heartRateCenter.hrCruiseEnabled = false
        } else {
            cruiseConfirmPrompt.show()
            cruiseConfirmPrompt.startTimeout(5)
        }
    }

    onBikeModePressed: {
        if(!workoutScreen.bikeModeEnabled){
            workoutScreen.showBikeModePrompt()
        }
    }

    CruiseConfirmPrompt {
        id: cruiseConfirmPrompt

        x: 300 ; y: 220
        visible: false
        onTimedOut: {
            hide()
        }

        onConfirmed: {
            heartRateCenter.hrCruiseEnabled = true
            hide()
        }
    }

    Connections {
        target: workoutScreen
        onWorkoutStopped: {
            brainboard.requestChangeFanSpeed(0)
        }
    }

    Connections {
        target: root.isSelected && !cruiseConfirmPrompt.visible && !workoutScreen.bikeModePromptShowing ? keymapper : null

        onEnter2Pressed: {
            if (main.blockKeys) return

            main.debugKeys("enter2Pressed", "ToolsView")
            switch (toolsMenu.currentItem.myData.name) {
                case "HRC Cruise":
                    if (toolsMenu.currentItem.hrcEnabled)
                        hrcCruisePressed()
                    break
                case "Bike Mode":
                    bikeModePressed()
                    break
                case "Fan":
                    fanButtonPressed()
                    break
                case "Begin Cool Down":
                    if (toolsMenu.currentItem.cooldownEnabled)
                        cooldownPressed()
                    break
                case "Change Units":
                    unitsPressed()
                    break
                default:
                    break

            }
        }

        onBackPressed: {
            if (main.blockKeys) return

            main.debugKeys("backPressed", "ToolsView")
            container.unselectView()
        }

        onDoubleLeftPressed: {
            main.debugKeys("doubleLeftPressed", "ToolsView")
            toolsMenu.jumpToPrev()
        }
        onDoubleRightPressed: {
            main.debugKeys("doubleRightPressed", "ToolsView")
            toolsMenu.jumpToNext()
        }
    }

    Image {
        id: background
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter

        source: "graphics/netpulse/DATA_gray shape_120612.png"
    }

    Text {
        id: headerText
        visible: root.isSelected

        anchors {
            top: parent.top
            topMargin: 25
            horizontalCenter: parent.horizontalCenter
        }
        color: "black"
        font { pixelSize: 36; bold: true }
        text: qsTr("use <img src=\"graphics/tft/REW_icon.png\" /> <img src=\"graphics/tft/FWD_icon.png\" /> to navigate TOOLS") + application.emptyString
    }

    // List Model of buttons
    ListModel {
        id: buttons

        ListElement { name: "Fan" ; label: QT_TR_NOOP("Fan") }
        ListElement { name: "Begin Cool Down" ; label: QT_TR_NOOP("Begin Cool Down") }
        ListElement { name: "Change Units" ; label: QT_TR_NOOP("Change Units") }
        ListElement { name: "HRC Cruise" ; label: QT_TR_NOOP("HRC Cruise")  }
    }

    ListModel {
        id: buttons_bike

        ListElement { name: "Fan" ; label: QT_TR_NOOP("Fan") }
        ListElement { name: "Begin Cool Down" ; label: QT_TR_NOOP("Begin Cool Down") }
        ListElement { name: "Bike Mode" ; label: QT_TR_NOOP("Bike Mode") }
        ListElement { name: "Change Units" ; label: QT_TR_NOOP("Change Units") }
        ListElement { name: "HRC Cruise" ; label: QT_TR_NOOP("HRC Cruise")  }
    }

    ListView {
        id: toolsMenu
        function jumpToPrev() {
            var wrap = function(idx) { return (idx - 1) < 0 ? toolsMenu.count - 1 : (idx - 1) };
            var jumpIndex = currentIndex
            for (var i = wrap(currentIndex);; i = wrap(i)) {
                if (contentItem.children[i + 1].isEnabled) {
                    jumpIndex = i
                    break
                }
            }
            currentIndex = jumpIndex
        }
        function jumpToNext() {
            var wrap = function(idx) { return (idx + 1) % toolsMenu.count };
            var jumpIndex = currentIndex
            for (var i = wrap(currentIndex);; i = wrap(i)) {
                if (contentItem.children[i + 1].isEnabled) {
                    jumpIndex = i
                    break
                }
            }
            currentIndex = jumpIndex
        }
        width: 865
        height: 225

        spacing: model.count === 5 ? 35 : 95
        orientation: ListView.Horizontal
        anchors { horizontalCenter: parent.horizontalCenter; verticalCenter: parent.verticalCenter }
        model: brainboard.productTypeString === "Bike" && root.bikeModeAvailable ? buttons_bike : buttons
        delegate: buttonDelegate
    }

    Component {
        id: buttonDelegate
        Item {
            property variant myData: model
            property bool fanHighlighted: name === "Fan" && brainboard.fanSpeed === 100
            property bool bikeModeHighlighted: name === "Bike Mode" && workoutScreen.bikeModeEnabled
            property bool hrcHighlighted: name === "HRC Cruise" && heartRateCenter.hrCruiseEnabled
            property bool cooldownHighlighted: name === "Begin Cool Down" && workoutManager.workoutMode === WorkoutRunData.COOL_DOWN_MODE
            property bool isCurrentItem: root.isSelected && ListView.isCurrentItem
            property bool hrcEnabled: heartRateCenter.cruiseAllowed || hrcHighlighted
            property bool cooldownEnabled: root.cooldownEnabled
            property bool isEnabled: !((name === "HRC Cruise" && !hrcEnabled) || (name === "Begin Cool Down" && !cooldownEnabled))
            height: 210
            width: 145

            Rectangle {

                color: "white"
                radius: 5
                anchors.fill: parent
                // TODO would be faster to use a static image here
                gradient: Gradient {
                    GradientStop { position: 0.0; color: isCurrentItem || hrcHighlighted || cooldownHighlighted ? "#ffd990" : "#f0f0f0" }
                    GradientStop { position: 1.0; color: isCurrentItem || hrcHighlighted || cooldownHighlighted ? "#ffd990" : "#a0a0a0" }
                }

                Image {
                    source: isCurrentItem || hrcHighlighted || cooldownHighlighted ? "graphics/netpulse/Button_Down_State_orange.png" : "graphics/netpulse/Button_Up_State_gray.png"
                    anchors.fill: parent
                    anchors.margins: 5
                    smooth: true

                    Image {
                        id: buttonImage
                        anchors {
                            bottom: buttonText.top
                            bottomMargin: 40
                            horizontalCenter: parent.horizontalCenter
                        }
                        source: {
                            switch (name) {
                            case "Begin Cool Down":
                                return "graphics/tft/toolicon_COOLDOWN.png"
                            case "Fan":
                                return fanHighlighted ? "graphics/tft/toolicon_FAN_GREEN.png" : "graphics/tft/toolicon_FAN.png"
                            case "Bike Mode":
                                return bikeModeHighlighted ? "graphics/tft/toolicon_BIKE_GREEN.png" : "graphics/tft/toolicon_BIKE.png"
                            case "Change Units":
                                return root.units == "METRIC" ? "graphics/tft/toolicon_UNITS_KM.png" : "graphics/tft/toolicon_UNITS_MI.png"
                            case "HRC Cruise":
                                return "graphics/tft/toolicon_HRC.png"
                            }
                        }
                        fillMode: Image.PreserveAspectFit
                        opacity: isEnabled ? 1.0 : 0.5
                    }

                    Text {
                        id: buttonText

                        anchors {
                            bottom: parent.bottom
                            bottomMargin: 15
                            horizontalCenter: parent.horizontalCenter
                        }
                        font { family: "arial"; pixelSize: 17; bold: true }
                        lineHeight: 0.9
                        width: parent.width - 30 ; height: 35 ;
                        horizontalAlignment: Text.AlignHCenter
                        verticalAlignment: Text.AlignTop
                        wrapMode: Text.WordWrap
                        text: qsTr(label) + application.emptyString
                        opacity: isEnabled ? 1.0 : 0.5
                    }
                }
            }
        }
    }

    Text {
        id: helpText
        visible: root.isSelected

        anchors {
            bottom: parent.bottom
            bottomMargin: 25
            horizontalCenter: parent.horizontalCenter
        }
        color: "black"
        font { pixelSize: 36; bold: true }
        text: qsTr("press ENTER to toggle") + application.emptyString
    }
}
